@@ -81,14 +81,26 @@ class WebHelpers(BrowserView):
 
     View name: @@webhelpers
     """
-    media_name = "++resource++euphorie.media"
-    resources_name = "++resource++euphorie.resources"
-    js_resources_name = "++resource++euphorie.resources"
-    bundle_name = "bundle.js"
-    bundle_name_min = "bundle.min.js"
+    certificates_path = "++resource++euphorie.resources/oira/certificates"
+    media_path = "++resource++euphorie.resources/media"
+    style_path = "++resource++euphorie.resources/oira/style"
+
+    css_path = "++resource++euphorie.resources/oira/style/all.css"
+    css_path_min = "++resource++euphorie.resources/oira/style/all.css"
+
+    js_path = "++resource++euphorie.resources/oira/script/bundle.js"
+    js_path_min = "++resource++euphorie.resources/oira/script/bundle.min.js"
+
     group_model = Group
     survey_session_model = SurveySession
 
+    @property
+    @memoize
+    def resources_timestamp(self):
+        return api.portal.get_registry_record(
+            "euphorie.deployment.resources_timestamp", default=""
+        )
+
     @property
     @memoize
     def server_timezone(self):
@@ -204,7 +216,9 @@ class WebHelpers(BrowserView):
         traversed_session = self.traversed_session
         return traversed_session.session.id if traversed_session else ""
 
-    def update_completion_percentage(self, session):
+    def update_completion_percentage(self, session=None):
+        if not session:
+            session = self.traversed_session.session
         query = (
             Session.query(SurveyTreeItem)
             .filter(SurveyTreeItem.session_id == session.id)
@@ -388,7 +402,7 @@ class WebHelpers(BrowserView):
     @memoize
     def client_url(self):
         """Return the absolute URL for the client."""
-        return self.client and self.client.absolute_url()
+        return self.client.absolute_url() if self.client else '.'
 
     @property
     @memoize
@@ -422,23 +436,41 @@ class WebHelpers(BrowserView):
             return self.client_url
         return self._base_url()
 
+    @property
+    @memoize
+    def certificates_url(self):
+        return "{}/{}".format(
+            self.client_url, self.certificates_path)
+
     @property
     @memoize
     def media_url(self):
         return "{}/{}".format(
-            self.client_url, self.media_name)
+            self.client_url, self.media_path)
 
     @property
     @memoize
-    def resources_url(self):
+    def style_url(self):
         return "{}/{}".format(
-            self.client_url, self.resources_name)
+            self.client_url, self.style_path)
 
     @property
     @memoize
-    def js_resources_url(self):
-        return "{}/{}".format(
-            self.client_url, self.js_resources_name)
+    def css_url(self):
+        return "{}/{}?t={}".format(
+            self.client_url,
+            self.css_path if not self.debug_mode else self.css_path_min,
+            self.resources_timestamp
+        )
+
+    @property
+    @memoize
+    def js_url(self):
+        return "{}/{}?t={}".format(
+            self.client_url,
+            self.js_path if not self.debug_mode else self.js_path_min,
+            self.resources_timestamp
+        )
 
     @property
     @memoize
@@ -903,56 +935,3 @@ class WebHelpers(BrowserView):
     def __call__(self):
         return self
 
-
-class Appendix(WebHelpers):
-    """ Browser View for showing the appendix with various links to
-    copyright, license, etc.
-    Since this is very client-specific, it gets its own template for easy
-    customisation.
-
-    """
-
-    def __call__(self):
-        return self
-
-
-class Logo(WebHelpers):
-    """ Browser View for showing the markup for the logo
-    Since this is very client-specific, it gets its own template for easy
-    customisation.
-
-    """
-
-    def __call__(self):
-        return self
-
-
-class UserMenu(WebHelpers):
-    """
-    View class for the User Menu
-
-    """
-
-    def __call__(self):
-        return self.index()
-
-
-class HelpMenu(WebHelpers):
-    """
-    View class for the User Menu
-
-    """
-
-    def __call__(self):
-        return self.index()
-
-
-class UpdateCompletionPercentage(WebHelpers):
-
-    percentage = 0
-
-    def __call__(self):
-        self.percentage = self.update_completion_percentage(
-            self.traversed_session.session
-        )
-        return self.index()
